--[[
	Name:		TargetArmor
	Version:	1.0
	Author:		Blackmail (knightchris2001@yahoo.com)
		Copyright (c) 2006 Christopher A. Florin (Blackmail on Arthus)
	Description:	Displays estimated armor value for targeted players.
	URL:		http://www.curse-gaming.com/mod.php?addid=3210
]]

local ARMOR = "ARMOR";
local MODNAME = "TARGETARMOR";
local MELEEPERCENT = "MeleePercent";
local MELEEHITS = "MeleeHits";
local RANGEDPERCENT = "RangedPercent";
local RANGEDHITS = "RangedHits";
local databaseversion = 1.0;  --increment this any time the database format changes


local AverageMeleeHit;
local AverageOffhandHit;
local AverageRangedHit;

local PlayerName;

local lEventHandler = {};

local TargetArmor_DAMAGEDATA =
    {
        { type = "Ranged", pattern = "Your Shoot Bow hits (.+) for (%d+).", target = 0, pts = 1 },
        { type = "Ranged", pattern = "Your Shoot Crossbow hits (.+) for (%d+).", target = 0, pts = 1 },
        { type = "Ranged", pattern = "Your Shoot Gun hits (.+) for (%d+).", target = 0, pts = 1 },
        { type = "Ranged", pattern = "Your Auto Shot hits (.+) for (%d+).", target = 0, pts = 1 },
        { type = "Melee", pattern = "You hit (.+) for (%d+).", target = 0, pts = 1 },

        { exit = true }
    };

--These variables are part of the genetic algorithm.  An explanation of the algorithm can be found at the bottom.

--OLDDATATHRESHOLD:
--  When TotalHitpointChange is equal to or greater than this number, half of all the previously collected data is
--      automatically assumed old and thrown out.
--  The default of 1000 is the equivalent of 10 complete health bars.
local OLDDATATHRESHOLD = 1000;
--PERCENTAGEMODIFIER:
--  The higher this number, an accurate reading will come faster, but it will adjust over time slower.
--  The lower this number, the slower an accurate readin will come, but it will adjust over time faster.
local PERCENTAGEMODIFIER = 0.98;

local function TargetArmor_Display()
    local targetname = UnitName("target");
    if ( not targetname ) then HideUIPanel(TargetArmorFrame); return; end

    local TotalRangedPercentage;
    local TotalRangedHits;
    local TotalMeleePercentage;
    local TotalMeleeHits;

    local playerlevel = UnitLevel("player");

    if ( UnitIsPlayer("target") ) then
        TotalRangedPercentage = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..RANGEDPERCENT, MODNAME);
        TotalRangedHits = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..RANGEDHITS, MODNAME);
        TotalMeleePercentage = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..MELEEPERCENT, MODNAME);
        TotalMeleeHits = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..MELEEHITS, MODNAME);
    else
        local targetlevel = UnitLevel("target");
        targetname = targetname..":"..targetlevel;

        TotalRangedPercentage = MobArmor[PlayerName][targetname].RANGEDPERCENT;
        TotalRangedHits = MobArmor[PlayerName][targetname].RANGEDHITS;
        TotalMeleePercentage = MobArmor[PlayerName][targetname].MELEEPERCENT;
        TotalMeleeHits = MobArmor[PlayerName][targetname].MELEEHITS;
    end

    if ( TotalRangedHits <= 1 and TotalMeleeHits <= 1) then getglobal("TargetArmorText"):SetText("NO DATA"); return; end
    local pr = 0;

    if ( TotalMeleeHits > 1 and TotalRangedHits > 1 ) then
        pr = 1 - ( TotalMeleePercentage + TotalRangedPercentage ) / ( TotalMeleeHits + TotalRangedHits );
    else
        if ( TotalMeleeHits <= 1 ) then
            pr = 1 - ( TotalRangedPercentage / TotalRangedHits );
        elseif ( TotalRangedHits <= 1 ) then
            pr = 1 - ( TotalMeleePercentage / TotalMeleeHits );
        else
            HideUIPanel(TargetArmorFrame);
            return;
        end
    end

    local ea = TargetArmor_PercentToArmor(pr, playerlevel);

    pr = floor(pr*10000+.5)/100;
    ea = floor(ea+.5);

    --Store player estimated armor rating in PlayerDB public vars.
    if ( UnitIsPlayer("target") ) then PlayerDB_setPlayerVar(targetname, ARMOR, ea); end

    ShowUIPanel(TargetArmorFrame);
    getglobal("TargetArmorText"):SetText("AC: "..ea.."/"..pr.."%");
end

--------------------------------------------------------------------------------------------------
-- Utility functions
--  * Neither of these functions attempt to do any rounding for you.
--------------------------------------------------------------------------------------------------
function TargetArmor_ArmorToPercent(armor, playerlevel)
    local ar = armor / ( 85 * playerlevel + 400 );
    return ar / ( ar + 1 );
end

function TargetArmor_PercentToArmor(percent, playerlevel)
    return ( percent * ( 85 * playerlevel + 400 ) ) / ( 1 - percent );
end

--------------------------------------------------------------------------------------------------
-- OnFoo functions
--------------------------------------------------------------------------------------------------
function TargetArmor_OnLoad()
	for key,value in lEventHandler do
		this:RegisterEvent(key);
	end

	if ( not MobArmor ) then
	    MobArmor = {};
	    MobArmor["version"] = databaseversion;
	end
end

function TargetArmor_OnEvent(event)
	if (lEventHandler[event]) then
		lEventHandler[event](arg1, arg2, arg3, arg4);
	end
end

lEventHandler["PLAYER_LOGIN"] = function ()
    PlayerName = UnitName("player");
    if ( not MobArmor[PlayerName] ) then MobArmor[PlayerName] = {}; end
end

lEventHandler["CHAT_MSG_COMBAT_SELF_HITS"] = function ()
    for index, value in TargetArmor_DAMAGEDATA do
        if ( value.exit ) then return; end

        local s, e;
        local results = { };
        s, e, results[0], results[1] = string.find(arg1, value.pattern);

        local match = false;
        if ( results[value.pts] and results[value.target] ) then match = true; end

        if ( match ) then
            local targetname = UnitName("target");

            if ( value.type == "Ranged" ) then
                --Do nothing; this is a melee event *SHOULD NOT HAPPEN*
            elseif ( value.type == "Melee" ) then
                --Do nothing until I can figure out how to distinguish between offhand and melee.
                local lowDmg, hiDmg, offlowDmg, offhiDmg, posBuff, negBuff, percentmod = UnitDamage("player");

                if ( ( offlowDmg ~= offhiDmg ) and ( offlowDmg ~= 0 or offhiDmg ~= 0 ) ) then return; end
                --if ( percentmod ~= 1 ) then return; end
                lowDmg = lowDmg * percentmod; hiDmg = hiDmg * percentmod;

                AverageMeleeHit = (lowDmg + hiDmg) / 2;    --Calculate the player's average melee hit

                local currenthit = results[value.pts];  --Find the value of the last hit
                local percentage = currenthit / AverageMeleeHit;

                local TotalMeleePercentage = 0;
                local TotalMeleeHits = 0;

                local OldDataModifier;

                --Retrieve old data
                if ( UnitIsPlayer("target") ) then
                    TotalMeleePercentage = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..MELEEPERCENT, MODNAME);
                    TotalMeleeHits = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..MELEEHITS, MODNAME);
                    OldDataModifier = PERCENTAGEMODIFIER;
                else
                    local targetlevel = UnitLevel("target");
                    targetname = targetname..":"..targetlevel;

                    TotalMeleePercentage = MobArmor[PlayerName][targetname].MELEEPERCENT;
                    TotalMeleeHits = MobArmor[PlayerName][targetname].MELEEHITS;

                    OldDataModifier = PERCENTAGEMODIFIER;    --Mobs do not have evolving armor ratings, but players gain levels
                end

                if ( TotalMeleePercentage and TotalMeleeHits ) then   --If data exists, modify it
                    if ( TotalMeleePercentage > OLDDATATHRESHOLD ) then
                        TotalMeleePercentage = TotalMeleePercentage / 2;
                        TotalMeleeHits = TotalMeleeHits / 2;
                    end

                    TotalMeleePercentage = TotalMeleePercentage*OldDataModifier + percentage;
                    TotalMeleeHits = TotalMeleeHits*OldDataModifier + 1;
                else    --Set new data since no data exists
                    TotalMeleePercentage = percentage;
                    TotalMeleeHits = 1;
                end

                --Store new data
                if ( UnitIsPlayer("target") ) then
                    PlayerDB_setPlayerModVar(targetname, PlayerName.."_"..MELEEPERCENT, TotalMeleePercentage, MODNAME);
                    PlayerDB_setPlayerModVar(targetname, PlayerName.."_"..MELEEHITS, TotalMeleeHits, MODNAME);
                else
                    MobArmor[PlayerName][targetname].MELEEPERCENT = TotalMeleePercentage;
                    MobArmor[PlayerName][targetname].MELEEHITS = TotalMeleeHits;
                end

                --Calculate percentage decrease
                TargetArmor_Display();
                return;
            else
                --DEFAULT_CHAT_FRAME:AddMessage(value.type);
            end
        end
    end
end

lEventHandler["CHAT_MSG_SPELL_SELF_DAMAGE"] = function ()
    for index, value in TargetArmor_DAMAGEDATA do
        if ( value.exit ) then return; end

        local s, e;
        local results = { };
        s, e, results[0], results[1] = string.find(arg1, value.pattern);

        local match = false;
        if ( results[value.pts] and results[value.target] ) then match = true; end

        if ( match ) then
            local targetname = UnitName("target");

            if ( value.type == "Ranged" ) then
                local speed, lowDmg, hiDmg = UnitRangedDamage("player");    --Yes these two lines are ripped from WowWiki
                AverageRangedHit = (lowDmg + hiDmg) / 2;    --Calculate the player's average ranged hit

                local currenthit = results[value.pts];  --Find the value of the last hit
                local percentage = currenthit / AverageRangedHit;

                local TotalRangedPercentage = 0;
                local TotalRangedHits = 0;

                local OldDataModifier;

                --Retrieve old data
                if ( UnitIsPlayer("target") ) then
                    TotalRangedPercentage = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..RANGEDPERCENT, MODNAME);
                    TotalRangedHits = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..RANGEDHITS, MODNAME);
                    OldDataModifier = PERCENTAGEMODIFIER;
                else
                    local targetlevel = UnitLevel("target");

                    targetname = targetname..":"..targetlevel;

                    TotalRangedPercentage = MobArmor[PlayerName][targetname].RANGEDPERCENT;
                    TotalRangedHits = MobArmor[PlayerName][targetname].RANGEDHITS;

                    OldDataModifier = PERCENTAGEMODIFIER;    --Mobs do not have evolving armor ratings, but players gain levels
                end

                if ( TotalRangedPercentage and TotalRangedHits ) then   --If data exists, modify it
                    if ( TotalRangedPercentage > OLDDATATHRESHOLD ) then
                        TotalRangedPercentage = TotalRangedPercentage / 2;
                        TotalRangedHits = TotalRangedHits / 2;
                    end

                    TotalRangedPercentage = TotalRangedPercentage*OldDataModifier + percentage;
                    TotalRangedHits = TotalRangedHits*OldDataModifier + 1;
                else    --Set new data since no data exists
                    TotalRangedPercentage = percentage;
                    TotalRangedHits = 1;
                end

                --Store new data
                if ( UnitIsPlayer("target") ) then
                    PlayerDB_setPlayerModVar(targetname, PlayerName.."_"..RANGEDPERCENT, TotalRangedPercentage, MODNAME);
                    PlayerDB_setPlayerModVar(targetname, PlayerName.."_"..RANGEDHITS, TotalRangedHits, MODNAME);
                else
                    MobArmor[PlayerName][targetname].RANGEDPERCENT = TotalRangedPercentage;
                    MobArmor[PlayerName][targetname].RANGEDHITS = TotalRangedHits;
                end

                --Calculate percentage decrease
                TargetArmor_Display();
                return;
            elseif ( value.type == "Melee" ) then
                --Do nothing, this is a Ranged event *SHOULD NOT HAPPEN*
            end
        end
    end
end

lEventHandler["PLAYER_TARGET_CHANGED"] = function ()
    if ( UnitIsPlayer("target") ) then
        local targetname = UnitName("target");

        --Retrieve old data
        local TotalRangedPercentage = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..RANGEDPERCENT, MODNAME);
        local TotalRangedHits = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..RANGEDHITS, MODNAME);
        local TotalMeleePercentage = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..MELEEPERCENT, MODNAME);
        local TotalMeleeHits = PlayerDB_getPlayerModVar(targetname, PlayerName.."_"..MELEEHITS, MODNAME);

        if ( TotalRangedPercentage and TotalRangedHits and TotalMeleePercentage and TotalMeleeHits ) then
            --Data exists; do nothing
        else
            --Set new data since no data exists
            PlayerDB_setPlayerModVar(targetname, PlayerName.."_"..RANGEDPERCENT, 0, MODNAME);
            PlayerDB_setPlayerModVar(targetname, PlayerName.."_"..RANGEDHITS, 0, MODNAME);
            PlayerDB_setPlayerModVar(targetname, PlayerName.."_"..MELEEPERCENT, 0, MODNAME);
            PlayerDB_setPlayerModVar(targetname, PlayerName.."_"..MELEEHITS, 0, MODNAME);
        end
    else
        local targetname = UnitName("target");

        if ( not targetname ) then return; end

        local targetlevel = UnitLevel("target");
        targetname = targetname..":"..targetlevel;

        if ( not MobArmor[PlayerName][targetname] ) then
            MobArmor[PlayerName][targetname] = {};
            MobArmor[PlayerName][targetname].MELEEPERCENT = 0;
            MobArmor[PlayerName][targetname].MELEEHITS = 0;
            MobArmor[PlayerName][targetname].RANGEDPERCENT = 0;
            MobArmor[PlayerName][targetname].RANGEDHITS = 0;
        else

        end
    end

     TargetArmor_Display();
end